# -*- coding: utf-8 -*-
"""Cisco DNA Center API wrappers.

Copyright (c) 2019-2021 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from past.types import basestring

from dnacentersdk.config import (
    DEFAULT_DEBUG,
    DEFAULT_VERSION,
    DEFAULT_BASE_URL,
    DEFAULT_SINGLE_REQUEST_TIMEOUT,
    DEFAULT_WAIT_ON_RATE_LIMIT,
    DEFAULT_VERIFY,
)
import dnacentersdk.environment as dnacenter_environment
from dnacentersdk.exceptions import AccessTokenError, VersionError
from dnacentersdk.models.mydict import mydict_data_factory
from dnacentersdk.models.schema_validator import SchemaValidator
from dnacentersdk.restsession import RestSession
from dnacentersdk.utils import check_type

from .authentication import Authentication
from .v1_2_10.clients import \
    Clients as Clients_v1_2_10
from .v1_2_10.command_runner import \
    CommandRunner as CommandRunner_v1_2_10
from .v1_2_10.devices import \
    Devices as Devices_v1_2_10
from .v1_2_10.fabric_wired import \
    FabricWired as FabricWired_v1_2_10
from .v1_2_10.file import \
    File as File_v1_2_10
from .v1_2_10.network_discovery import \
    NetworkDiscovery as NetworkDiscovery_v1_2_10
from .v1_2_10.networks import \
    Networks as Networks_v1_2_10
from .v1_2_10.non_fabric_wireless import \
    NonFabricWireless as NonFabricWireless_v1_2_10
from .v1_2_10.path_trace import \
    PathTrace as PathTrace_v1_2_10
from .v1_2_10.pnp import \
    Pnp as Pnp_v1_2_10
from .v1_2_10.swim import \
    Swim as Swim_v1_2_10
from .v1_2_10.site_profile import \
    SiteProfile as SiteProfile_v1_2_10
from .v1_2_10.sites import \
    Sites as Sites_v1_2_10
from .v1_2_10.tag import \
    Tag as Tag_v1_2_10
from .v1_2_10.task import \
    Task as Task_v1_2_10
from .v1_2_10.template_programmer import \
    TemplateProgrammer as TemplateProgrammer_v1_2_10
from .v1_3_0.clients import \
    Clients as Clients_v1_3_0
from .v1_3_0.command_runner import \
    CommandRunner as CommandRunner_v1_3_0
from .v1_3_0.devices import \
    Devices as Devices_v1_3_0
from .v1_3_0.fabric_wired import \
    FabricWired as FabricWired_v1_3_0
from .v1_3_0.file import \
    File as File_v1_3_0
from .v1_3_0.network_discovery import \
    NetworkDiscovery as NetworkDiscovery_v1_3_0
from .v1_3_0.networks import \
    Networks as Networks_v1_3_0
from .v1_3_0.non_fabric_wireless import \
    NonFabricWireless as NonFabricWireless_v1_3_0
from .v1_3_0.path_trace import \
    PathTrace as PathTrace_v1_3_0
from .v1_3_0.pnp import \
    Pnp as Pnp_v1_3_0
from .v1_3_0.swim import \
    Swim as Swim_v1_3_0
from .v1_3_0.site_profile import \
    SiteProfile as SiteProfile_v1_3_0
from .v1_3_0.sites import \
    Sites as Sites_v1_3_0
from .v1_3_0.tag import \
    Tag as Tag_v1_3_0
from .v1_3_0.task import \
    Task as Task_v1_3_0
from .v1_3_0.template_programmer import \
    TemplateProgrammer as TemplateProgrammer_v1_3_0
from .v1_3_1.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v1_3_1
from .v1_3_1.clients import \
    Clients as Clients_v1_3_1
from .v1_3_1.command_runner import \
    CommandRunner as CommandRunner_v1_3_1
from .v1_3_1.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v1_3_1
from .v1_3_1.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v1_3_1
from .v1_3_1.devices import \
    Devices as Devices_v1_3_1
from .v1_3_1.event_management import \
    EventManagement as EventManagement_v1_3_1
from .v1_3_1.fabric_wired import \
    FabricWired as FabricWired_v1_3_1
from .v1_3_1.file import \
    File as File_v1_3_1
from .v1_3_1.issues import \
    Issues as Issues_v1_3_1
from .v1_3_1.network_discovery import \
    NetworkDiscovery as NetworkDiscovery_v1_3_1
from .v1_3_1.network_settings import \
    NetworkSettings as NetworkSettings_v1_3_1
from .v1_3_1.non_fabric_wireless import \
    NonFabricWireless as NonFabricWireless_v1_3_1
from .v1_3_1.path_trace import \
    PathTrace as PathTrace_v1_3_1
from .v1_3_1.site_design import \
    SiteDesign as SiteDesign_v1_3_1
from .v1_3_1.sites import \
    Sites as Sites_v1_3_1
from .v1_3_1.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v1_3_1
from .v1_3_1.tag import \
    Tag as Tag_v1_3_1
from .v1_3_1.task import \
    Task as Task_v1_3_1
from .v1_3_1.topology import \
    Topology as Topology_v1_3_1
from .v1_3_1.users import \
    Users as Users_v1_3_1
from .v1_3_3.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v1_3_3
from .v1_3_3.clients import \
    Clients as Clients_v1_3_3
from .v1_3_3.command_runner import \
    CommandRunner as CommandRunner_v1_3_3
from .v1_3_3.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v1_3_3
from .v1_3_3.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v1_3_3
from .v1_3_3.devices import \
    Devices as Devices_v1_3_3
from .v1_3_3.discovery import \
    Discovery as Discovery_v1_3_3
from .v1_3_3.event_management import \
    EventManagement as EventManagement_v1_3_3
from .v1_3_3.file import \
    File as File_v1_3_3
from .v1_3_3.issues import \
    Issues as Issues_v1_3_3
from .v1_3_3.network_settings import \
    NetworkSettings as NetworkSettings_v1_3_3
from .v1_3_3.path_trace import \
    PathTrace as PathTrace_v1_3_3
from .v1_3_3.sda import \
    Sda as Sda_v1_3_3
from .v1_3_3.site_design import \
    SiteDesign as SiteDesign_v1_3_3
from .v1_3_3.sites import \
    Sites as Sites_v1_3_3
from .v1_3_3.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v1_3_3
from .v1_3_3.tag import \
    Tag as Tag_v1_3_3
from .v1_3_3.task import \
    Task as Task_v1_3_3
from .v1_3_3.topology import \
    Topology as Topology_v1_3_3
from .v1_3_3.users import \
    Users as Users_v1_3_3
from .v1_3_3.wireless import \
    Wireless as Wireless_v1_3_3
from .v2_1_1.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_1_1
from .v2_1_1.clients import \
    Clients as Clients_v2_1_1
from .v2_1_1.command_runner import \
    CommandRunner as CommandRunner_v2_1_1
from .v2_1_1.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_1_1
from .v2_1_1.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_1_1
from .v2_1_1.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_1_1
from .v2_1_1.devices import \
    Devices as Devices_v2_1_1
from .v2_1_1.discovery import \
    Discovery as Discovery_v2_1_1
from .v2_1_1.event_management import \
    EventManagement as EventManagement_v2_1_1
from .v2_1_1.file import \
    File as File_v2_1_1
from .v2_1_1.itsm import \
    Itsm as Itsm_v2_1_1
from .v2_1_1.issues import \
    Issues as Issues_v2_1_1
from .v2_1_1.network_settings import \
    NetworkSettings as NetworkSettings_v2_1_1
from .v2_1_1.path_trace import \
    PathTrace as PathTrace_v2_1_1
from .v2_1_1.sda import \
    Sda as Sda_v2_1_1
from .v2_1_1.site_design import \
    SiteDesign as SiteDesign_v2_1_1
from .v2_1_1.sites import \
    Sites as Sites_v2_1_1
from .v2_1_1.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_1_1
from .v2_1_1.tag import \
    Tag as Tag_v2_1_1
from .v2_1_1.task import \
    Task as Task_v2_1_1
from .v2_1_1.topology import \
    Topology as Topology_v2_1_1
from .v2_1_1.users import \
    Users as Users_v2_1_1
from .v2_1_1.wireless import \
    Wireless as Wireless_v2_1_1
from .v2_1_2.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_1_2
from .v2_1_2.applications import \
    Applications as Applications_v2_1_2
from .v2_1_2.clients import \
    Clients as Clients_v2_1_2
from .v2_1_2.command_runner import \
    CommandRunner as CommandRunner_v2_1_2
from .v2_1_2.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_1_2
from .v2_1_2.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_1_2
from .v2_1_2.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_1_2
from .v2_1_2.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_1_2
from .v2_1_2.devices import \
    Devices as Devices_v2_1_2
from .v2_1_2.discovery import \
    Discovery as Discovery_v2_1_2
from .v2_1_2.event_management import \
    EventManagement as EventManagement_v2_1_2
from .v2_1_2.file import \
    File as File_v2_1_2
from .v2_1_2.itsm import \
    Itsm as Itsm_v2_1_2
from .v2_1_2.issues import \
    Issues as Issues_v2_1_2
from .v2_1_2.network_settings import \
    NetworkSettings as NetworkSettings_v2_1_2
from .v2_1_2.path_trace import \
    PathTrace as PathTrace_v2_1_2
from .v2_1_2.sda import \
    Sda as Sda_v2_1_2
from .v2_1_2.sensors import \
    Sensors as Sensors_v2_1_2
from .v2_1_2.site_design import \
    SiteDesign as SiteDesign_v2_1_2
from .v2_1_2.sites import \
    Sites as Sites_v2_1_2
from .v2_1_2.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_1_2
from .v2_1_2.tag import \
    Tag as Tag_v2_1_2
from .v2_1_2.task import \
    Task as Task_v2_1_2
from .v2_1_2.topology import \
    Topology as Topology_v2_1_2
from .v2_1_2.users import \
    Users as Users_v2_1_2
from .v2_1_2.wireless import \
    Wireless as Wireless_v2_1_2
from .v2_2_1.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_2_1
from .v2_2_1.applications import \
    Applications as Applications_v2_2_1
from .v2_2_1.clients import \
    Clients as Clients_v2_2_1
from .v2_2_1.command_runner import \
    CommandRunner as CommandRunner_v2_2_1
from .v2_2_1.compliance import \
    Compliance as Compliance_v2_2_1
from .v2_2_1.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_2_1
from .v2_2_1.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_2_1
from .v2_2_1.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_2_1
from .v2_2_1.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_2_1
from .v2_2_1.devices import \
    Devices as Devices_v2_2_1
from .v2_2_1.discovery import \
    Discovery as Discovery_v2_2_1
from .v2_2_1.event_management import \
    EventManagement as EventManagement_v2_2_1
from .v2_2_1.file import \
    File as File_v2_2_1
from .v2_2_1.itsm import \
    Itsm as Itsm_v2_2_1
from .v2_2_1.issues import \
    Issues as Issues_v2_2_1
from .v2_2_1.network_settings import \
    NetworkSettings as NetworkSettings_v2_2_1
from .v2_2_1.path_trace import \
    PathTrace as PathTrace_v2_2_1
from .v2_2_1.reports import \
    Reports as Reports_v2_2_1
from .v2_2_1.sda import \
    Sda as Sda_v2_2_1
from .v2_2_1.security_advisories import \
    SecurityAdvisories as SecurityAdvisories_v2_2_1
from .v2_2_1.sensors import \
    Sensors as Sensors_v2_2_1
from .v2_2_1.site_design import \
    SiteDesign as SiteDesign_v2_2_1
from .v2_2_1.sites import \
    Sites as Sites_v2_2_1
from .v2_2_1.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_2_1
from .v2_2_1.tag import \
    Tag as Tag_v2_2_1
from .v2_2_1.task import \
    Task as Task_v2_2_1
from .v2_2_1.topology import \
    Topology as Topology_v2_2_1
from .v2_2_1.users import \
    Users as Users_v2_2_1
from .v2_2_1.wireless import \
    Wireless as Wireless_v2_2_1
from .v2_2_2_3.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_2_2_3
from .v2_2_2_3.applications import \
    Applications as Applications_v2_2_2_3
from .v2_2_2_3.authentication_management import \
    AuthenticationManagement as AuthenticationManagement_v2_2_2_3
from .v2_2_2_3.clients import \
    Clients as Clients_v2_2_2_3
from .v2_2_2_3.command_runner import \
    CommandRunner as CommandRunner_v2_2_2_3
from .v2_2_2_3.compliance import \
    Compliance as Compliance_v2_2_2_3
from .v2_2_2_3.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_2_2_3
from .v2_2_2_3.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_2_2_3
from .v2_2_2_3.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_2_2_3
from .v2_2_2_3.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_2_2_3
from .v2_2_2_3.devices import \
    Devices as Devices_v2_2_2_3
from .v2_2_2_3.discovery import \
    Discovery as Discovery_v2_2_2_3
from .v2_2_2_3.event_management import \
    EventManagement as EventManagement_v2_2_2_3
from .v2_2_2_3.file import \
    File as File_v2_2_2_3
from .v2_2_2_3.health_and_performance import \
    HealthAndPerformance as HealthAndPerformance_v2_2_2_3
from .v2_2_2_3.itsm import \
    Itsm as Itsm_v2_2_2_3
from .v2_2_2_3.issues import \
    Issues as Issues_v2_2_2_3
from .v2_2_2_3.licenses import \
    Licenses as Licenses_v2_2_2_3
from .v2_2_2_3.network_settings import \
    NetworkSettings as NetworkSettings_v2_2_2_3
from .v2_2_2_3.path_trace import \
    PathTrace as PathTrace_v2_2_2_3
from .v2_2_2_3.reports import \
    Reports as Reports_v2_2_2_3
from .v2_2_2_3.sda import \
    Sda as Sda_v2_2_2_3
from .v2_2_2_3.security_advisories import \
    SecurityAdvisories as SecurityAdvisories_v2_2_2_3
from .v2_2_2_3.platform_configuration import \
    PlatformConfiguration as PlatformConfiguration_v2_2_2_3
from .v2_2_2_3.sensors import \
    Sensors as Sensors_v2_2_2_3
from .v2_2_2_3.site_design import \
    SiteDesign as SiteDesign_v2_2_2_3
from .v2_2_2_3.sites import \
    Sites as Sites_v2_2_2_3
from .v2_2_2_3.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_2_2_3
from .v2_2_2_3.tag import \
    Tag as Tag_v2_2_2_3
from .v2_2_2_3.task import \
    Task as Task_v2_2_2_3
from .v2_2_2_3.topology import \
    Topology as Topology_v2_2_2_3
from .v2_2_2_3.users import \
    Users as Users_v2_2_2_3
from .v2_2_2_3.wireless import \
    Wireless as Wireless_v2_2_2_3
from .v2_2_3_3.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_2_3_3
from .v2_2_3_3.applications import \
    Applications as Applications_v2_2_3_3
from .v2_2_3_3.platform_configuration import \
    PlatformConfiguration as PlatformConfiguration_v2_2_3_3
from .v2_2_3_3.authentication_management import \
    AuthenticationManagement as AuthenticationManagement_v2_2_3_3
from .v2_2_3_3.clients import \
    Clients as Clients_v2_2_3_3
from .v2_2_3_3.command_runner import \
    CommandRunner as CommandRunner_v2_2_3_3
from .v2_2_3_3.compliance import \
    Compliance as Compliance_v2_2_3_3
from .v2_2_3_3.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_2_3_3
from .v2_2_3_3.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_2_3_3
from .v2_2_3_3.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_2_3_3
from .v2_2_3_3.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_2_3_3
from .v2_2_3_3.devices import \
    Devices as Devices_v2_2_3_3
from .v2_2_3_3.disaster_recovery import \
    DisasterRecovery as DisasterRecovery_v2_2_3_3
from .v2_2_3_3.discovery import \
    Discovery as Discovery_v2_2_3_3
from .v2_2_3_3.event_management import \
    EventManagement as EventManagement_v2_2_3_3
from .v2_2_3_3.fabric_wireless import \
    FabricWireless as FabricWireless_v2_2_3_3
from .v2_2_3_3.file import \
    File as File_v2_2_3_3
from .v2_2_3_3.health_and_performance import \
    HealthAndPerformance as HealthAndPerformance_v2_2_3_3
from .v2_2_3_3.itsm import \
    Itsm as Itsm_v2_2_3_3
from .v2_2_3_3.issues import \
    Issues as Issues_v2_2_3_3
from .v2_2_3_3.licenses import \
    Licenses as Licenses_v2_2_3_3
from .v2_2_3_3.network_settings import \
    NetworkSettings as NetworkSettings_v2_2_3_3
from .v2_2_3_3.path_trace import \
    PathTrace as PathTrace_v2_2_3_3
from .v2_2_3_3.policy import \
    Policy as Policy_v2_2_3_3
from .v2_2_3_3.reports import \
    Reports as Reports_v2_2_3_3
from .v2_2_3_3.sda import \
    Sda as Sda_v2_2_3_3
from .v2_2_3_3.security_advisories import \
    SecurityAdvisories as SecurityAdvisories_v2_2_3_3
from .v2_2_3_3.sensors import \
    Sensors as Sensors_v2_2_3_3
from .v2_2_3_3.site_design import \
    SiteDesign as SiteDesign_v2_2_3_3
from .v2_2_3_3.sites import \
    Sites as Sites_v2_2_3_3
from .v2_2_3_3.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_2_3_3
from .v2_2_3_3.tag import \
    Tag as Tag_v2_2_3_3
from .v2_2_3_3.task import \
    Task as Task_v2_2_3_3
from .v2_2_3_3.topology import \
    Topology as Topology_v2_2_3_3
from .v2_2_3_3.users import \
    Users as Users_v2_2_3_3
from .v2_2_3_3.wireless import \
    Wireless as Wireless_v2_2_3_3
from .v2_3_3_0.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_3_3_0
from .v2_3_3_0.applications import \
    Applications as Applications_v2_3_3_0
from .v2_3_3_0.cisco_dna_center_system import \
    CiscoDnaCenterSystem as CiscoDnaCenterSystem_v2_3_3_0
from .v2_3_3_0.clients import \
    Clients as Clients_v2_3_3_0
from .v2_3_3_0.command_runner import \
    CommandRunner as CommandRunner_v2_3_3_0
from .v2_3_3_0.compliance import \
    Compliance as Compliance_v2_3_3_0
from .v2_3_3_0.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_3_3_0
from .v2_3_3_0.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_3_3_0
from .v2_3_3_0.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_3_3_0
from .v2_3_3_0.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_3_3_0
from .v2_3_3_0.devices import \
    Devices as Devices_v2_3_3_0
from .v2_3_3_0.platform_configuration import \
    PlatformConfiguration as PlatformConfiguration_v2_3_3_0
from .v2_3_3_0.discovery import \
    Discovery as Discovery_v2_3_3_0
from .v2_3_3_0.event_management import \
    EventManagement as EventManagement_v2_3_3_0
from .v2_3_3_0.fabric_wireless import \
    FabricWireless as FabricWireless_v2_3_3_0
from .v2_3_3_0.file import \
    File as File_v2_3_3_0
from .v2_3_3_0.health_and_performance import \
    HealthAndPerformance as HealthAndPerformance_v2_3_3_0
from .v2_3_3_0.itsm import \
    Itsm as Itsm_v2_3_3_0
from .v2_3_3_0.issues import \
    Issues as Issues_v2_3_3_0
from .v2_3_3_0.lan_automation import \
    LanAutomation as LanAutomation_v2_3_3_0
from .v2_3_3_0.licenses import \
    Licenses as Licenses_v2_3_3_0
from .v2_3_3_0.network_settings import \
    NetworkSettings as NetworkSettings_v2_3_3_0
from .v2_3_3_0.path_trace import \
    PathTrace as PathTrace_v2_3_3_0
from .v2_3_3_0.reports import \
    Reports as Reports_v2_3_3_0
from .v2_3_3_0.sda import \
    Sda as Sda_v2_3_3_0
from .v2_3_3_0.security_advisories import \
    SecurityAdvisories as SecurityAdvisories_v2_3_3_0
from .v2_3_3_0.sensors import \
    Sensors as Sensors_v2_3_3_0
from .v2_3_3_0.site_design import \
    SiteDesign as SiteDesign_v2_3_3_0
from .v2_3_3_0.sites import \
    Sites as Sites_v2_3_3_0
from .v2_3_3_0.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_3_3_0
from .v2_3_3_0.system_settings import \
    SystemSettings as SystemSettings_v2_3_3_0
from .v2_3_3_0.tag import \
    Tag as Tag_v2_3_3_0
from .v2_3_3_0.task import \
    Task as Task_v2_3_3_0
from .v2_3_3_0.topology import \
    Topology as Topology_v2_3_3_0
from .v2_3_3_0.users import \
    Users as Users_v2_3_3_0
from .v2_3_3_0.wireless import \
    Wireless as Wireless_v2_3_3_0
from .v2_3_5_3.application_policy import \
    ApplicationPolicy as ApplicationPolicy_v2_3_5_3
from .v2_3_5_3.applications import \
    Applications as Applications_v2_3_5_3
from .v2_3_5_3.authentication_management import \
    AuthenticationManagement as AuthenticationManagement_v2_3_5_3
from .v2_3_5_3.cisco_dna_center_system import \
    CiscoDnaCenterSystem as CiscoDnaCenterSystem_v2_3_5_3
from .v2_3_5_3.clients import \
    Clients as Clients_v2_3_5_3
from .v2_3_5_3.command_runner import \
    CommandRunner as CommandRunner_v2_3_5_3
from .v2_3_5_3.compliance import \
    Compliance as Compliance_v2_3_5_3
from .v2_3_5_3.configuration_archive import \
    ConfigurationArchive as ConfigurationArchive_v2_3_5_3
from .v2_3_5_3.configuration_templates import \
    ConfigurationTemplates as ConfigurationTemplates_v2_3_5_3
from .v2_3_5_3.device_onboarding_pnp import \
    DeviceOnboardingPnp as DeviceOnboardingPnp_v2_3_5_3
from .v2_3_5_3.device_replacement import \
    DeviceReplacement as DeviceReplacement_v2_3_5_3
from .v2_3_5_3.devices import \
    Devices as Devices_v2_3_5_3
from .v2_3_5_3.discovery import \
    Discovery as Discovery_v2_3_5_3
from .v2_3_5_3.eox import \
    Eox as Eox_v2_3_5_3
from .v2_3_5_3.event_management import \
    EventManagement as EventManagement_v2_3_5_3
from .v2_3_5_3.fabric_wireless import \
    FabricWireless as FabricWireless_v2_3_5_3
from .v2_3_5_3.file import \
    File as File_v2_3_5_3
from .v2_3_5_3.health_and_performance import \
    HealthAndPerformance as HealthAndPerformance_v2_3_5_3
from .v2_3_5_3.itsm import \
    Itsm as Itsm_v2_3_5_3
from .v2_3_5_3.itsm_integration import \
    ItsmIntegration as ItsmIntegration_v2_3_5_3
from .v2_3_5_3.issues import \
    Issues as Issues_v2_3_5_3
from .v2_3_5_3.lan_automation import \
    LanAutomation as LanAutomation_v2_3_5_3
from .v2_3_5_3.licenses import \
    Licenses as Licenses_v2_3_5_3
from .v2_3_5_3.network_settings import \
    NetworkSettings as NetworkSettings_v2_3_5_3
from .v2_3_5_3.path_trace import \
    PathTrace as PathTrace_v2_3_5_3
from .v2_3_5_3.platform import \
    Platform as Platform_v2_3_5_3
from .v2_3_5_3.reports import \
    Reports as Reports_v2_3_5_3
from .v2_3_5_3.sda import \
    Sda as Sda_v2_3_5_3
from .v2_3_5_3.security_advisories import \
    SecurityAdvisories as SecurityAdvisories_v2_3_5_3
from .v2_3_5_3.sensors import \
    Sensors as Sensors_v2_3_5_3
from .v2_3_5_3.site_design import \
    SiteDesign as SiteDesign_v2_3_5_3
from .v2_3_5_3.sites import \
    Sites as Sites_v2_3_5_3
from .v2_3_5_3.software_image_management_swim import \
    SoftwareImageManagementSwim as SoftwareImageManagementSwim_v2_3_5_3
from .v2_3_5_3.system_settings import \
    SystemSettings as SystemSettings_v2_3_5_3
from .v2_3_5_3.tag import \
    Tag as Tag_v2_3_5_3
from .v2_3_5_3.task import \
    Task as Task_v2_3_5_3
from .v2_3_5_3.topology import \
    Topology as Topology_v2_3_5_3
from .v2_3_5_3.user_and_roles import \
    UserandRoles as UserandRoles_v2_3_5_3
from .v2_3_5_3.users import \
    Users as Users_v2_3_5_3
from .v2_3_5_3.wireless import \
    Wireless as Wireless_v2_3_5_3
from .custom_caller import CustomCaller


class DNACenterAPI(object):
    """Cisco DNA Center API wrapper.

    Creates a 'session' for all API calls through a created DNACenterAPI
    object.  The 'session' handles authentication, provides the needed headers,
    and checks all responses for error conditions.

    DNACenterAPI wraps all of the individual DNA Center APIs and represents
    them in a simple hierarchical structure.
    """

    def __init__(self, username=None,
                 password=None,
                 encoded_auth=None,
                 base_url=None,
                 single_request_timeout=None,
                 wait_on_rate_limit=None,
                 verify=None,
                 version=None,
                 debug=None,
                 object_factory=mydict_data_factory,
                 validator=SchemaValidator):
        """Create a new DNACenterAPI object.
        An access token is required to interact with the DNA Center APIs.
        This package supports two methods for you to generate the
        authorization token:

          1. Provide a encoded_auth value (username:password encoded in
          base 64). *This has priority over the following method*

          2. Provide username and password values.

        This package supports two methods for you to set those values:

          1. Provide the parameter. That is the encoded_auth or
          username and password parameters.

          2. If an argument is not supplied, the package checks for
          its environment variable counterpart. That is the
          DNA_CENTER_ENCODED_AUTH, DNA_CENTER_USERNAME,
          DNA_CENTER_PASSWORD.

        When not given enough parameters an AccessTokenError is raised.

        Args:
            base_url(basestring): The base URL to be prefixed to the
                individual API endpoint suffixes.
                Defaults to the DNA_CENTER_BASE_URL environment variable or
                dnacentersdk.config.DEFAULT_BASE_URL
                if the environment variable is not set.
            username(basestring): HTTP Basic Auth username.
            password(basestring): HTTP Basic Auth password.
            encoded_auth(basestring): HTTP Basic Auth base64 encoded string.
            single_request_timeout(int): Timeout (in seconds) for RESTful HTTP
                requests. Defaults to the DNA_CENTER_SINGLE_REQUEST_TIMEOUT
                environment variable or
                dnacentersdk.config.DEFAULT_SINGLE_REQUEST_TIMEOUT
                if the environment variable is not set.
            wait_on_rate_limit(bool): Enables or disables automatic rate-limit
                handling. Defaults to the DNA_CENTER_WAIT_ON_RATE_LIMIT
                environment variable or
                dnacentersdk.config.DEFAULT_WAIT_ON_RATE_LIMIT
                if the environment variable is not set.
            verify(bool,basestring): Controls whether we verify the server's
                TLS certificate, or a string, in which case it must be a path
                to a CA bundle to use. Defaults to the DNA_CENTER_VERIFY 
                environment variable or
                dnacentersdk.config.DEFAULT_VERIFY if the environment
                variables are not set.
            version(basestring): Controls which version of DNA_CENTER to use.
                Defaults to the DNA_CENTER_VERSION environment variable or
                dnacentersdk.config.DEFAULT_VERSION
                if the environment variable is not set.
            debug(bool,basestring): Controls whether to log information about
                DNA Center APIs' request and response process.
                Defaults to the DNA_CENTER_DEBUG environment variable or False
                if the environment variable is not set.
            object_factory(callable): The factory function to use to create
                Python objects from the returned DNA Center JSON data objects.
            validator(callable): The factory function to use to validate
                Python objects sent in the body of the request.

        Returns:
            DNACenterAPI: A new DNACenterAPI object.

        Raises:
            TypeError: If the parameter types are incorrect.
            AccessTokenError: If an access token is not provided via the
                access_token argument or an environment variable.
            VersionError: If the version is not provided via the version
                argument or an environment variable, or it is not a
                DNA Center API supported version
                ['1.2.10', '1.3.0', '1.3.1', '1.3.3', '2.1.1', '2.1.2',
                '2.2.1', '2.2.2.3', '2.2.3.3',
                '2.3.3.0', '2.3.5.3'].

        """
        username = username or dnacenter_environment.get_env_username()
        password = password or dnacenter_environment.get_env_password()
        encoded_auth = encoded_auth or dnacenter_environment.get_env_encoded_auth()
        base_url = base_url or dnacenter_environment.get_env_base_url() or DEFAULT_BASE_URL

        if single_request_timeout is None:
            single_request_timeout = dnacenter_environment.get_env_single_request_timeout() or DEFAULT_SINGLE_REQUEST_TIMEOUT

        if wait_on_rate_limit is None:
            wait_on_rate_limit = dnacenter_environment.get_env_wait_on_rate_limit() or DEFAULT_WAIT_ON_RATE_LIMIT

        if verify is None:
            verify = dnacenter_environment.get_env_verify() if dnacenter_environment.get_env_verify() != None else DEFAULT_VERIFY

        version = version or dnacenter_environment.get_env_version() or DEFAULT_VERSION

        if debug is None:
            debug = dnacenter_environment.get_env_debug() or DEFAULT_DEBUG

        check_type(base_url, basestring)
        check_type(single_request_timeout, int)
        check_type(wait_on_rate_limit, bool)
        check_type(debug, (bool, basestring), may_be_none=True)
        check_type(username, basestring, may_be_none=True)
        check_type(password, basestring, may_be_none=True)
        check_type(encoded_auth, basestring, may_be_none=True)
        check_type(verify, (bool, basestring), may_be_none=False)
        check_type(version, basestring, may_be_none=False)

        if version not in ['1.2.10', '1.3.0', '1.3.1', '1.3.3', '2.1.1',
                           '2.1.2', '2.2.1',
                           '2.2.2.3', '2.2.3.3',
                           '2.3.3.0', '2.3.5.3']:
            raise VersionError(
                'Unknown API version, '
                + 'known versions are {}'.format(
                    '1.2.10, 1.3.0, 1.3.1, 1.3.3, 2.1.1, 2.1.2, 2.2.1, 2.2.2.3,'
                    + '2.2.3.3, 2.3.3.0 and 2.3.5.3.'
                )
            )

        if isinstance(debug, str):
            debug = 'true' in debug.lower()

        # Init Authentication wrapper early to use for basicAuth requests
        self.authentication = Authentication(
            base_url, object_factory,
            single_request_timeout=single_request_timeout,
            verify=verify,
        )

        # Check if the user has provided the required basicAuth parameters
        if encoded_auth is None and (username is None or password is None):
            raise AccessTokenError(
                "You need an access token to interact with the DNA Center"
                " APIs. DNA Center uses HTTP Basic Auth to create an access"
                " token. You must provide the username and password or just"
                " the encoded_auth, either by setting each parameter or its"
                " environment variable counterpart ("
                "DNA_CENTER_USERNAME, DNA_CENTER_PASSWORD,"
                " DNA_CENTER_ENCODED_AUTH)."
            )

        def get_access_token():
            return self.authentication.authentication_api(
                username=username,
                password=password,
                encoded_auth=encoded_auth).Token

        # Create the API session
        # All of the API calls associated with a DNACenterAPI object will
        # leverage a single RESTful 'session' connecting to the DNA Center
        # cloud.
        self._session = RestSession(
            get_access_token=get_access_token,
            access_token=get_access_token(),
            base_url=base_url,
            single_request_timeout=single_request_timeout,
            wait_on_rate_limit=wait_on_rate_limit,
            verify=verify,
            version=version,
            debug=debug,
        )

        _validator = validator(version).json_schema_validate

        # API wrappers
        if version == '1.2.10':
            self.clients = \
                Clients_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.fabric_wired = \
                FabricWired_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.network_discovery = \
                NetworkDiscovery_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.networks = \
                Networks_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.non_fabric_wireless = \
                NonFabricWireless_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.pnp = \
                Pnp_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.swim = \
                Swim_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.site_profile = \
                SiteProfile_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v1_2_10(
                    self._session, object_factory, _validator
                )
            self.template_programmer = \
                TemplateProgrammer_v1_2_10(
                    self._session, object_factory, _validator
                )
        if version == '1.3.0':
            self.clients = \
                Clients_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.fabric_wired = \
                FabricWired_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.network_discovery = \
                NetworkDiscovery_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.networks = \
                Networks_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.non_fabric_wireless = \
                NonFabricWireless_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.pnp = \
                Pnp_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.swim = \
                Swim_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.site_profile = \
                SiteProfile_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v1_3_0(
                    self._session, object_factory, _validator
                )
            self.template_programmer = \
                TemplateProgrammer_v1_3_0(
                    self._session, object_factory, _validator
                )
        if version == '1.3.1':
            self.application_policy = \
                ApplicationPolicy_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.fabric_wired = \
                FabricWired_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.network_discovery = \
                NetworkDiscovery_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.non_fabric_wireless = \
                NonFabricWireless_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v1_3_1(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v1_3_1(
                    self._session, object_factory, _validator
                )
        if version == '1.3.3':
            self.application_policy = \
                ApplicationPolicy_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v1_3_3(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v1_3_3(
                    self._session, object_factory, _validator
                )
        if version == '2.1.1':
            self.application_policy = \
                ApplicationPolicy_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_1_1(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_1_1(
                    self._session, object_factory, _validator
                )
        if version == '2.1.2':
            self.application_policy = \
                ApplicationPolicy_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_1_2(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_1_2(
                    self._session, object_factory, _validator
                )
        if version == '2.2.1':
            self.application_policy = \
                ApplicationPolicy_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.compliance = \
                Compliance_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.reports = \
                Reports_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.security_advisories = \
                SecurityAdvisories_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_2_1(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_2_1(
                    self._session, object_factory, _validator
                )
        if version == '2.2.2.3':
            self.application_policy = \
                ApplicationPolicy_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.authentication_management = \
                AuthenticationManagement_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.compliance = \
                Compliance_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.health_and_performance = \
                HealthAndPerformance_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.licenses = \
                Licenses_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.platform_configuration = \
                PlatformConfiguration_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.reports = \
                Reports_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.security_advisories = \
                SecurityAdvisories_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_2_2_3(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_2_2_3(
                    self._session, object_factory, _validator
                )
        if version == '2.2.3.3':
            self.application_policy = \
                ApplicationPolicy_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.authentication_management = \
                AuthenticationManagement_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.compliance = \
                Compliance_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.disaster_recovery = \
                DisasterRecovery_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.fabric_wireless = \
                FabricWireless_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.health_and_performance = \
                HealthAndPerformance_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.licenses = \
                Licenses_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.platform_configuration = \
                PlatformConfiguration_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.policy = \
                Policy_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.reports = \
                Reports_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.security_advisories = \
                SecurityAdvisories_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_2_3_3(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_2_3_3(
                    self._session, object_factory, _validator
                )
        if version == '2.3.3.0':
            self.application_policy = \
                ApplicationPolicy_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.cisco_dna_center_system = \
                CiscoDnaCenterSystem_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.compliance = \
                Compliance_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.fabric_wireless = \
                FabricWireless_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.health_and_performance = \
                HealthAndPerformance_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.lan_automation = \
                LanAutomation_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.licenses = \
                Licenses_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.platform_configuration = \
                PlatformConfiguration_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.reports = \
                Reports_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.security_advisories = \
                SecurityAdvisories_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.system_settings = \
                SystemSettings_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_3_3_0(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_3_3_0(
                    self._session, object_factory, _validator
                )
        if version == '2.3.5.3':
            self.application_policy = \
                ApplicationPolicy_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.applications = \
                Applications_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.authentication_management = \
                AuthenticationManagement_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.cisco_dna_center_system = \
                CiscoDnaCenterSystem_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.clients = \
                Clients_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.command_runner = \
                CommandRunner_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.compliance = \
                Compliance_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.configuration_archive = \
                ConfigurationArchive_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.configuration_templates = \
                ConfigurationTemplates_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.device_onboarding_pnp = \
                DeviceOnboardingPnp_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.device_replacement = \
                DeviceReplacement_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.devices = \
                Devices_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.discovery = \
                Discovery_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.eox = \
                Eox_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.event_management = \
                EventManagement_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.fabric_wireless = \
                FabricWireless_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.file = \
                File_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.health_and_performance = \
                HealthAndPerformance_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.itsm = \
                Itsm_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.itsm_integration = \
                ItsmIntegration_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.issues = \
                Issues_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.lan_automation = \
                LanAutomation_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.licenses = \
                Licenses_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.network_settings = \
                NetworkSettings_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.path_trace = \
                PathTrace_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.platform = \
                Platform_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.platform_configuration = \
                Platform_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.reports = \
                Reports_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.sda = \
                Sda_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.security_advisories = \
                SecurityAdvisories_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.sensors = \
                Sensors_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.site_design = \
                SiteDesign_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.sites = \
                Sites_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.software_image_management_swim = \
                SoftwareImageManagementSwim_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.system_settings = \
                SystemSettings_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.tag = \
                Tag_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.task = \
                Task_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.topology = \
                Topology_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.user_and_roles = \
                UserandRoles_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.users = \
                Users_v2_3_5_3(
                    self._session, object_factory, _validator
                )
            self.wireless = \
                Wireless_v2_3_5_3(
                    self._session, object_factory, _validator
                )
        self.custom_caller = \
            CustomCaller(self._session, object_factory)

    @property
    def session(self):
        """The DNA Center API session."""
        return self._session

    @property
    def access_token(self):
        """The access token used for API calls to the DNA Center service."""
        return self._session.access_token

    @property
    def base_url(self):
        """The base URL prefixed to the individual API endpoint suffixes."""
        return self._session.base_url

    @property
    def single_request_timeout(self):
        """Timeout (in seconds) for an single HTTP request."""
        return self._session.single_request_timeout

    @property
    def wait_on_rate_limit(self):
        """Automatic rate-limit handling enabled / disabled."""
        return self._session.wait_on_rate_limit

    @property
    def verify(self):
        """The verify (TLS Certificate) for the API endpoints."""
        return self._session._verify

    @property
    def version(self):
        """The API version of DNA Center."""
        return self._session._version

    @verify.setter
    def verify(self, value):
        """The verify (TLS Certificate) for the API endpoints."""
        self.authentication.verify = value
        self._session.verify = value

    @base_url.setter
    def base_url(self, value):
        """The base URL for the API endpoints."""
        self._session.base_url = value

    @single_request_timeout.setter
    def single_request_timeout(self, value):
        """The timeout (seconds) for a single HTTP REST API request."""
        self.authentication.single_request_timeout = value
        self._session.single_request_timeout = value

    @wait_on_rate_limit.setter
    def wait_on_rate_limit(self, value):
        """Enable or disable automatic rate-limit handling."""
        self._session.wait_on_rate_limit = value



# -*- coding: utf-8 -*-
"""Cisco DNA Center Network Settings API wrapper.

Copyright (c) 2019-2021 Cisco Systems.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

from builtins import *

from past.builtins import basestring

from ...restsession import RestSession
from ...utils import (
    apply_path_params,
    check_type,
    dict_from_items_with_values,
    dict_of_str,
)


class NetworkSettings(object):
    """Cisco DNA Center Network Settings API (version: 2.1.2).

    Wraps the DNA Center Network Settings
    API and exposes the API as native Python
    methods that return native Python objects.

    """

    def __init__(self, session, object_factory, request_validator):
        """Initialize a new NetworkSettings
        object with the provided RestSession.

        Args:
            session(RestSession): The RESTful session object to be used for
                API calls to the DNA Center service.

        Raises:
            TypeError: If the parameter types are incorrect.

        """
        check_type(session, RestSession)

        super(NetworkSettings, self).__init__()

        self._session = session
        self._object_factory = object_factory
        self._request_validator = request_validator

    def update_global_pool(self,
                           settings=None,
                           headers=None,
                           payload=None,
                           active_validation=True,
                           **request_parameters):
        """API to update global pool.

        Args:
            settings(object): Settings, property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        _payload = {
            'settings':
                settings,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_03b4c8b44919b964_v2_1_2')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/global-pool')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload)

        return self._object_factory('bpm_03b4c8b44919b964_v2_1_2', json_data)

    def delete_global_ip_pool(self,
                              id,
                              headers=None,
                              **request_parameters):
        """API to delete global IP pool.

        Args:
            id(basestring): global pool id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(id, basestring,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/global-pool/${id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(endpoint_full_url, params=_params,
                                             headers=_headers)
        else:
            json_data = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_1eaa8b2148ab81de_v2_1_2', json_data)

    def delete_device_credential(self,
                                 id,
                                 headers=None,
                                 **request_parameters):
        """Delete device credential.

        Args:
            id(basestring): global credential id.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(id, basestring,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'id': id,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/device-credential/${id}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(endpoint_full_url, params=_params,
                                             headers=_headers)
        else:
            json_data = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_259eab3045988958_v2_1_2', json_data)

    def get_network(self,
                    site_id=None,
                    headers=None,
                    **request_parameters):
        """API to get  DHCP and DNS center server details.

        Args:
            site_id(basestring): Site id to get the network settings associated with the site.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(site_id, basestring)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
            'siteId':
                site_id,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/network')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_38b7eb13449b9471_v2_1_2', json_data)

    def delete_sp_profile(self,
                          sp_profile_name,
                          headers=None,
                          **request_parameters):
        """API to delete Service Provider profile (QoS).

        Args:
            sp_profile_name(basestring): sp profile name.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(sp_profile_name, basestring,
                   may_be_none=False)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'sp-profile-name': sp_profile_name,
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/sp-profile/${sp-profile-name}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.delete(endpoint_full_url, params=_params,
                                             headers=_headers)
        else:
            json_data = self._session.delete(endpoint_full_url, params=_params)

        return self._object_factory('bpm_4ca2db1143ebb5d7_v2_1_2', json_data)

    def update_sp_profile(self,
                          settings=None,
                          headers=None,
                          payload=None,
                          active_validation=True,
                          **request_parameters):
        """API to update SP profile.

        Args:
            settings(object): Settings, property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        _payload = {
            'settings':
                settings,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_5087daae4cc98566_v2_1_2')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/service-provider')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload)

        return self._object_factory('bpm_5087daae4cc98566_v2_1_2', json_data)

    def get_service_provider_details(self,
                                     headers=None,
                                     **request_parameters):
        """API to get service provider details (QoS).

        Args:
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/service-provider')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_70847bdc4d89a437_v2_1_2', json_data)

    def update_device_credentials(self,
                                  settings=None,
                                  headers=None,
                                  payload=None,
                                  active_validation=True,
                                  **request_parameters):
        """API to update device credentials.

        Args:
            settings(object): Settings, property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        _payload = {
            'settings':
                settings,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_4f947a1c4fc884f6_v2_1_2')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/device-credential')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload)

        return self._object_factory('bpm_4f947a1c4fc884f6_v2_1_2', json_data)

    def assign_credential_to_site(self,
                                  site_id,
                                  cliId=None,
                                  httpRead=None,
                                  httpWrite=None,
                                  snmpV2ReadId=None,
                                  snmpV2WriteId=None,
                                  snmpV3Id=None,
                                  headers=None,
                                  payload=None,
                                  active_validation=True,
                                  **request_parameters):
        """Assign Device Credential To Site .

        Args:
            cliId(string): Cli Id, property of the request body.
            httpRead(string): Http Read, property of the request body.
            httpWrite(string): Http Write, property of the request body.
            snmpV2ReadId(string): Snmp V2 Read Id, property of the request body.
            snmpV2WriteId(string): Snmp V2 Write Id, property of the request body.
            snmpV3Id(string): Snmp V3 Id, property of the request body.
            site_id(basestring): site id to assign credential.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(site_id, basestring,
                   may_be_none=False)
        if headers is not None:
            if '__persistbapioutput' in headers:
                check_type(headers.get('__persistbapioutput'),
                           basestring, may_be_none=False)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'siteId': site_id,
        }

        _payload = {
            'cliId':
                cliId,
            'snmpV2ReadId':
                snmpV2ReadId,
            'snmpV2WriteId':
                snmpV2WriteId,
            'httpRead':
                httpRead,
            'httpWrite':
                httpWrite,
            'snmpV3Id':
                snmpV3Id,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_4da91a544e29842d_v2_1_2')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/credential-to-site/${siteId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_4da91a544e29842d_v2_1_2', json_data)

    def update_network(self,
                       site_id,
                       settings=None,
                       headers=None,
                       payload=None,
                       active_validation=True,
                       **request_parameters):
        """API to update network for DHCP and DNS center server settings.

        Args:
            settings(object): Settings, property of the request body.
            site_id(basestring): Site id to update the network settings which is associated with the site.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(site_id, basestring,
                   may_be_none=False)
        if headers is not None:
            if '__persistbapioutput' in headers:
                check_type(headers.get('__persistbapioutput'),
                           bool)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'siteId': site_id,
        }

        _payload = {
            'settings':
                settings,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_698bfbb44dcb9fca_v2_1_2')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/network/${siteId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload,
                                          headers=_headers)
        else:
            json_data = self._session.put(endpoint_full_url, params=_params,
                                          json=_payload)

        return self._object_factory('bpm_698bfbb44dcb9fca_v2_1_2', json_data)

    def create_sp_profile(self,
                          settings=None,
                          headers=None,
                          payload=None,
                          active_validation=True,
                          **request_parameters):
        """API to create service provider profile(QOS).

        Args:
            settings(object): settings, property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        _payload = {
            'settings':
                settings,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_a39a1a214debb781_v2_1_2')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/service-provider')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_a39a1a214debb781_v2_1_2', json_data)

    def get_global_pool(self,
                        limit=None,
                        offset=None,
                        headers=None,
                        **request_parameters):
        """API to get global pool.

        Args:
            offset(basestring): offset/starting row.
            limit(basestring): No of Global Pools to be retrieved.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(offset, basestring)
        check_type(limit, basestring)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
            'offset':
                offset,
            'limit':
                limit,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/global-pool')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_c0bca85643c8b58d_v2_1_2', json_data)

    def get_device_credential_details(self,
                                      site_id=None,
                                      headers=None,
                                      **request_parameters):
        """API to get device credential details.

        Args:
            site_id(basestring): Site id to retrieve the credential details associated with the site.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(site_id, basestring)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
            'siteId':
                site_id,
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/device-credential')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.get(endpoint_full_url, params=_params,
                                          headers=_headers)
        else:
            json_data = self._session.get(endpoint_full_url, params=_params)

        return self._object_factory('bpm_899f08e7401b82dd_v2_1_2', json_data)

    def create_network(self,
                       site_id,
                       settings=None,
                       headers=None,
                       payload=None,
                       active_validation=True,
                       **request_parameters):
        """API to create a network for DHCP and DNS center server settings.

        Args:
            settings(object): Settings, property of the request body.
            site_id(basestring): Site id to which site details to associate with the network settings.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        check_type(site_id, basestring,
                   may_be_none=False)
        if headers is not None:
            if '__persistbapioutput' in headers:
                check_type(headers.get('__persistbapioutput'),
                           bool)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
            'siteId': site_id,
        }

        _payload = {
            'settings':
                settings,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_be892bd84a78865a_v2_1_2')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/network/${siteId}')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_be892bd84a78865a_v2_1_2', json_data)

    def create_device_credentials(self,
                                  settings=None,
                                  headers=None,
                                  payload=None,
                                  active_validation=True,
                                  **request_parameters):
        """API to create device credentials.

        Args:
            settings(object): Settings, property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        _payload = {
            'settings':
                settings,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_fbb95b37484a9fce_v2_1_2')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/device-credential')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_fbb95b37484a9fce_v2_1_2', json_data)

    def create_global_pool(self,
                           settings=None,
                           headers=None,
                           payload=None,
                           active_validation=True,
                           **request_parameters):
        """API to create global pool.

        Args:
            settings(object): Settings, property of the request body.
            headers(dict): Dictionary of HTTP Headers to send with the Request
                .
            payload(dict): A JSON serializable Python object to send in the
                body of the Request.
            active_validation(bool): Enable/Disable payload validation.
                Defaults to True.
            **request_parameters: Additional request parameters (provides
                support for parameters that may be added in the future).

        Returns:
            MyDict: JSON response. Access the object's properties by using
            the dot notation or the bracket notation.

        Raises:
            TypeError: If the parameter types are incorrect.
            MalformedRequest: If the request body created is invalid.
            ApiError: If the DNA Center cloud returns an error.
        """
        check_type(headers, dict)
        check_type(payload, dict)
        if headers is not None:
            if '__persistbapioutput' in headers:
                check_type(headers.get('__persistbapioutput'),
                           basestring)
            if 'X-Auth-Token' in headers:
                check_type(headers.get('X-Auth-Token'),
                           basestring, may_be_none=False)

        _params = {
        }
        _params.update(request_parameters)
        _params = dict_from_items_with_values(_params)

        path_params = {
        }

        _payload = {
            'settings':
                settings,
        }
        _payload.update(payload or {})
        _payload = dict_from_items_with_values(_payload)
        if active_validation:
            self._request_validator('jsd_f793192a43dabed9_v2_1_2')\
                .validate(_payload)

        with_custom_headers = False
        _headers = self._session.headers or {}
        if headers:
            _headers.update(dict_of_str(headers))
            with_custom_headers = True

        e_url = ('/dna/intent/api/v1/global-pool')
        endpoint_full_url = apply_path_params(e_url, path_params)
        if with_custom_headers:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload,
                                           headers=_headers)
        else:
            json_data = self._session.post(endpoint_full_url, params=_params,
                                           json=_payload)

        return self._object_factory('bpm_f793192a43dabed9_v2_1_2', json_data)
